@@ -69,7 +69,7 @@ public:
     muse::Ret exportScoreElements(const muse::io::path_t& in, const muse::io::path_t& out, const std::string& optionsJson,
                                   const OpenParams& openParams = {}) override;
 
-    muse::Ret exportScoreVideo(const muse::io::path_t& in, const muse::io::path_t& out) override;
+    muse::Ret exportScoreVideo(const muse::io::path_t& in, const muse::io::path_t& out, const OpenParams& openParams = {}) override;
 
     muse::Ret updateSource(const muse::io::path_t& in, const std::string& newSource, bool forceMode = false) override;
 
