@@ -676,6 +676,7 @@ void Segment::add(EngravingItem* el)
 
     track_idx_t track = el->track();
     assert(track != muse::nidx);
+    assert(track < score()->ntracks());
     assert(el->score() == score());
     assert(score()->nstaves() * VOICES == m_elist.size());
     // make sure offset is correct for staff
@@ -2737,23 +2738,27 @@ void Segment::addArticulationsToShape(const Chord* chord, Shape& shape)
     };
 
     for (Articulation* art : chord->articulations()) {
-        if (art->isOrnament()) {
-            Chord* cueNoteChord = toOrnament(art)->cueNoteChord();
-            if (cueNoteChord && cueNoteChord->upNote()->visible()) {
-                shape.add(cueNoteChord->shape().translate(cueNoteChord->pos() + cueNoteChord->staffOffset()));
+        if (art->isOrnament() || !art->addToSkyline()) {
+            continue;
+        }
+        shape.add(art->shape().translated(art->pos() + chord->pos()));
+        if (art->isTapping()) {
+            if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurAbove()) {
+                addTappingHalfSlurToShape(halfSlur);
             }
-        } else if (art->addToSkyline()) {
-            shape.add(art->shape().translated(art->pos() + chord->pos()));
-            if (art->isTapping()) {
-                if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurAbove()) {
-                    addTappingHalfSlurToShape(halfSlur);
-                }
-                if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurBelow()) {
-                    addTappingHalfSlurToShape(halfSlur);
-                }
+            if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurBelow()) {
+                addTappingHalfSlurToShape(halfSlur);
             }
         }
     }
+
+    Ornament* ornament = chord->findOrnament();
+    if (ornament) {
+        Chord* cueNoteChord = ornament->cueNoteChord();
+        if (cueNoteChord && cueNoteChord->upNote()->addToSkyline()) {
+            shape.add(cueNoteChord->shape().translate(cueNoteChord->pos() + cueNoteChord->staffOffset()));
+        }
+    }
 }
 
 //---------------------------------------------------------
