@@ -399,11 +399,13 @@ void Tuplet::startEditDrag(EditData& ed)
 
 void Tuplet::editDrag(EditData& ed)
 {
-    if (ed.curGrip == Grip::START) {
+    if (ed.curGrip == Grip::START || ed.curGrip == Grip::MIDDLE) {
         m_userP1 += ed.delta;
-    } else {
+    }
+    if (ed.curGrip == Grip::END || ed.curGrip == Grip::MIDDLE) {
         m_userP2 += ed.delta;
     }
+
     setGenerated(false);
     //layout();
     //score()->setUpdateAll();
@@ -414,10 +416,23 @@ void Tuplet::editDrag(EditData& ed)
 //   gripsPositions
 //---------------------------------------------------------
 
+int Tuplet::gripsCount() const
+{
+    return m_hasBracket ? 3 : 0;
+}
+
 std::vector<PointF> Tuplet::gripsPositions(const EditData&) const
 {
+    IF_ASSERT_FAILED(gripsCount() != 0) {
+        return std::vector<PointF>();
+    }
+
     const PointF pp(pagePos());
-    return { pp + m_p1, pp + m_p2 };
+    PointF left = pp + m_p1;
+    PointF right = pp + m_p2;
+    PointF middle = 0.5 * (left + right);
+
+    return { left, right, middle };
 }
 
 //---------------------------------------------------------
