@@ -66,8 +66,12 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "tubular-bells", { SoundId::Chimes, SoundCategory::Percussions } },
         { "carillon", { SoundId::Carillon, SoundCategory::Percussions, { SoundSubCategory::Metal } } },
         { "tuned-gongs", { SoundId::Gong, SoundCategory::Percussions, { SoundSubCategory::Metal } } },
-        { "hand-bells", { SoundId::Bell, SoundCategory::Percussions, { SoundSubCategory::Metal,
-                                                                       SoundSubCategory::Hand } } },
+        { "opera-gong", { SoundId::Gong, SoundCategory::Percussions, { SoundSubCategory::Metal,
+                                                                       SoundSubCategory::Opera } } },
+        { "wind-gong", { SoundId::Gong, SoundCategory::Percussions, { SoundSubCategory::Metal,
+                                                                      SoundSubCategory::Wind } } },
+        { "hand-bells", { SoundId::Bell, SoundCategory::Percussions, { SoundSubCategory::Hand } } },
+        { "hand-chimes", { SoundId::Chimes, SoundCategory::Percussions, { SoundSubCategory::Hand } } },
 
         { "orff-soprano-glockenspiel", { SoundId::Glockenspiel, SoundCategory::Percussions, { SoundSubCategory::Soprano,
                                                                                               SoundSubCategory::Orff } } },
@@ -104,8 +108,35 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "bass-drum", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Bass } } },
         { "snare-drum", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Snare } } },
         { "tom-toms", { SoundId::TomToms, SoundCategory::Percussions } },
+        { "ko-tsuzumi", { SoundId::KoTsuzumi, SoundCategory::Percussions } },
+        { "o-tsuzumi", { SoundId::OTsuzumi, SoundCategory::Percussions } },
+        { "kakko", { SoundId::Kakko, SoundCategory::Percussions } },
+        { "shime-daiko", { SoundId::ShimeDaiko, SoundCategory::Percussions } },
+        { "miya-daiko", { SoundId::MiyaDaiko, SoundCategory::Percussions } },
+        { "tsuri-daiko", { SoundId::TsuriDaiko, SoundCategory::Percussions } },
+        { "okedo-daiko", { SoundId::OkedoDaiko, SoundCategory::Percussions } },
+        { "kane", { SoundId::Kane, SoundCategory::Percussions } },
+        { "shoko", { SoundId::Shoko, SoundCategory::Percussions } },
         { "congas", { SoundId::Conga, SoundCategory::Percussions } },
+        { "xiaogu", { SoundId::Xiaogu, SoundCategory::Percussions } },
+        { "bangu", { SoundId::Bangu, SoundCategory::Percussions } },
+        { "dagu", { SoundId::Dagu, SoundCategory::Percussions } },
+        { "daluo", { SoundId::Daluo, SoundCategory::Percussions } },
+        { "xiaoluo", { SoundId::Xiaoluo, SoundCategory::Percussions } },
+        { "ban", { SoundId::Ban, SoundCategory::Percussions } },
+        { "dabo", { SoundId::Dabo, SoundCategory::Percussions } },
+        { "dabo", { SoundId::Dabo, SoundCategory::Percussions } },
+        { "naobo", { SoundId::Naobo, SoundCategory::Percussions } },
+        { "bangzi", { SoundId::Bangzi, SoundCategory::Percussions } },
+        { "janggu", { SoundId::Janggu, SoundCategory::Percussions } },
+        { "buk", { SoundId::Buk, SoundCategory::Percussions } },
+        { "sogo", { SoundId::Sogo, SoundCategory::Percussions } },
+        { "kkwaenggwari", { SoundId::Kkwaenggwari, SoundCategory::Percussions } },
+        { "jing", { SoundId::Jing, SoundCategory::Percussions } },
+        { "djembe", { SoundId::Djembe, SoundCategory::Percussions } },
+        { "doumbek", { SoundId::Doumbek, SoundCategory::Percussions } },
         { "cuica", { SoundId::Cuica, SoundCategory::Percussions } },
+        { "cajon", { SoundId::Cajon, SoundCategory::Percussions } },
         { "drumset", { SoundId::Drumset, SoundCategory::Percussions } },
         { "drum-kit-4", { SoundId::Drumset, SoundCategory::Percussions, { SoundSubCategory::FourPiece } } },
         { "drum-kit-5", { SoundId::Drumset, SoundCategory::Percussions, { SoundSubCategory::FivePiece } } },
@@ -125,10 +156,12 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "bowl-gongs", { SoundId::Gong, SoundCategory::Percussions, { SoundSubCategory::Metal,
                                                                        SoundSubCategory::Bowl } } },
         { "chains", { SoundId::Chain, SoundCategory::Percussions, { SoundSubCategory::Metal } } },
-        { "chinese-cymbal", { SoundId::Cymbal, SoundCategory::Percussions, { SoundSubCategory::Metal,
-                                                                             SoundSubCategory::Chinese } } },
+        { "china-cymbal", { SoundId::Cymbal, SoundCategory::Percussions, { SoundSubCategory::Metal,
+                                                                           SoundSubCategory::China } } },
         { "cowbell", { SoundId::Bell, SoundCategory::Percussions, { SoundSubCategory::Metal,
                                                                     SoundSubCategory::Cow } } },
+        { "agogo-bells", { SoundId::Bell, SoundCategory::Percussions, { SoundSubCategory::Metal,
+                                                                        SoundSubCategory::Agogo } } },
         { "crash-cymbal", { SoundId::Cymbal, SoundCategory::Percussions, { SoundSubCategory::Metal,
                                                                            SoundSubCategory::Crash } } },
         { "cymbal", { SoundId::Cymbal, SoundCategory::Percussions, { SoundSubCategory::Metal } } },
@@ -156,6 +189,9 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "guiro", { SoundId::Guiro, SoundCategory::Percussions, { SoundSubCategory::Wooden } } },
         { "temple-blocks", { SoundId::Block, SoundCategory::Percussions, { SoundSubCategory::Wooden,
                                                                            SoundSubCategory::Temple } } },
+        { "log-drum", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Wooden,
+                                                                     SoundSubCategory::Log } } },
+        { "ocean-drum", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Ocean } } },
         { "wood-blocks", { SoundId::Block, SoundCategory::Percussions, { SoundSubCategory::Wooden } } },
         { "wooden-wind-chimes", { SoundId::Chimes, SoundCategory::Percussions, { SoundSubCategory::Wooden,
                                                                                  SoundSubCategory::Wind } } },
@@ -167,6 +203,7 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "quijada", { SoundId::Quijada, SoundCategory::Percussions } },
         { "ratchet", { SoundId::Ratchet, SoundCategory::Percussions } },
         { "shaker", { SoundId::Shaker, SoundCategory::Percussions } },
+        { "shekere", { SoundId::Shekere, SoundCategory::Percussions } },
         { "sandpaper-blocks", { SoundId::Block, SoundCategory::Percussions, { SoundSubCategory::Sandpaper } } },
         { "glass-wind-chimes", { SoundId::Chimes, SoundCategory::Percussions, { SoundSubCategory::Glass,
                                                                                 SoundSubCategory::Wind } } },
@@ -178,12 +215,17 @@ PlaybackSetupData PercussionsSetupDataResolver::doResolve(const Instrument* inst
         { "tubo", { SoundId::Tubo, SoundCategory::Percussions } },
         { "vibraslap", { SoundId::Vibraslap, SoundCategory::Percussions } },
         { "whip", { SoundId::Whip, SoundCategory::Percussions } },
+        { "cannon", { SoundId::Cannon, SoundCategory::Percussions } },
+        { "bird-call", { SoundId::BirdCall, SoundCategory::Percussions } },
 
         { "marching-snare", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Marching,
                                                                            SoundSubCategory::Snare } } },
         { "marching-tenor-drums", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Marching,
                                                                                  SoundSubCategory::Snare,
                                                                                  SoundSubCategory::Tenor } } },
+        { "marching-show-tenors", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Show_Style,
+                                                                                 SoundSubCategory::Snare,
+                                                                                 SoundSubCategory::Tenor } } },
         { "marching-bass-drums", { SoundId::Drum, SoundCategory::Percussions, { SoundSubCategory::Marching,
                                                                                 SoundSubCategory::Snare,
                                                                                 SoundSubCategory::Bass } } },
