@@ -357,6 +357,10 @@ QPoint QWidgetAdapter::pos() const
 void QWidgetAdapter::show()
 {
     setVisible(true);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+    QShowEvent ev;
+    event(&ev);
+#endif
 }
 
 void QWidgetAdapter::setFixedHeight(int height)
@@ -622,6 +626,21 @@ void QWidgetAdapter::move(int x, int y)
 
 void QWidgetAdapter::setSize(QSize size)
 {
+#ifdef Q_OS_MACOS
+    FloatingWindow *floatingWindow = this->floatingWindow();
+    if (!floatingWindow) {
+        return;
+    }
+
+    const auto newSize = size.expandedTo(minimumSize());
+
+    QWindow* window = floatingWindow->windowHandle();
+    if (window && window->size() != newSize) {
+        QRect windowGeo = window->geometry();
+        windowGeo.setSize(newSize);
+        window->setGeometry(windowGeo);
+    }
+#endif
     QQuickItem::setSize(QSizeF(size));
 }
 
