@@ -30,6 +30,7 @@
 
 #include "dockcentralview.h"
 #include "dockpageview.h"
+#include "dockpanelview.h"
 #include "dockstatusbarview.h"
 #include "docktoolbarview.h"
 #include "dockingholderview.h"
@@ -85,6 +86,22 @@ static void clearRegistry()
 }
 }
 
+class DockWindow::UniqueConnectionHolder : public QObject
+{
+    Q_OBJECT
+public:
+    UniqueConnectionHolder(DockPageView* page, DockWindow* parent)
+        : QObject(parent), m_page(page) {}
+
+    void alignTopLevelToolBars()
+    {
+        static_cast<DockWindow*>(parent())->alignTopLevelToolBars(m_page);
+    }
+
+private:
+    DockPageView* m_page = nullptr;
+};
+
 DockWindow::DockWindow(QQuickItem* parent)
     : QQuickItem(parent), muse::Injectable(muse::iocCtxForQmlObject(this)),
     m_toolBars(this),
@@ -95,6 +112,12 @@ DockWindow::DockWindow(QQuickItem* parent)
 DockWindow::~DockWindow()
 {
     dockWindowProvider()->deinit();
+
+    // Without this, the connections would be deleted by the QObject destructor,
+    // because they are child objects of this. But since they use DockWindow-
+    // specific code (rather than QObject-specific), we need to delete them
+    // before the end of the DockWindow destructor.
+    qDeleteAll(m_pageConnections);
 }
 
 void DockWindow::componentComplete()
@@ -110,7 +133,9 @@ void DockWindow::componentComplete()
     connect(qApp, &QCoreApplication::aboutToQuit, this, &DockWindow::onQuit);
     connect(this, &QQuickItem::windowChanged, this, &DockWindow::windowPropertyChanged);
 
-    connect(this, &QQuickItem::widthChanged, this, &DockWindow::adjustContentForAvailableSpace);
+    connect(this, &QQuickItem::widthChanged, this, [this]() {
+        adjustContentForAvailableSpace(m_currentPage);
+    });
 }
 
 void DockWindow::geometryChange(const QRectF& newGeometry, const QRectF& oldGeometry)
@@ -123,7 +148,7 @@ void DockWindow::geometryChange(const QRectF& newGeometry, const QRectF& oldGeom
     //! NOTE: it is important to reset the current minimum width for all top-level toolbars
     //! Otherwise, the window content can be displaced after LayoutWidget::onResize(QSize newSize)
     //! due to lack of free space
-    QList<DockToolBarView*> topToolBars = topLevelToolBars(m_currentPage);
+    const QList<DockToolBarView*> topToolBars = topLevelToolBars(m_currentPage);
     for (DockToolBarView* toolBar : topToolBars) {
         toolBar->setMinimumWidth(toolBar->contentWidth());
     }
@@ -226,12 +251,10 @@ void DockWindow::loadPage(const QString& uri, const QVariantMap& params)
     if (isFirstOpening) {
         async::Async::call(this, [this, notifyAboutPageLoaded]() {
             if (!m_hasGeometryBeenRestored
-                || (m_mainWindow->windowHandle()->windowStates() & QWindow::FullScreen)) {
+                || (m_mainWindow->windowHandle()->windowStates() & Qt::WindowFullScreen)) {
                 //! NOTE: show window as maximized if no geometry has been restored
                 //! or if the user had closed app in FullScreen mode
                 m_mainWindow->windowHandle()->showMaximized();
-            } else {
-                m_mainWindow->windowHandle()->setVisible(true);
             }
 
             notifyAboutPageLoaded();
@@ -241,9 +264,9 @@ void DockWindow::loadPage(const QString& uri, const QVariantMap& params)
     }
 }
 
-bool DockWindow::isDockOpenAndCurrentInFrame(const QString& dockName) const
+bool DockWindow::isDockOpen(const QString& dockName) const
 {
-    return m_currentPage && m_currentPage->isDockOpenAndCurrentInFrame(dockName);
+    return m_currentPage && m_currentPage->isDockOpen(dockName);
 }
 
 void DockWindow::toggleDock(const QString& dockName)
@@ -367,11 +390,14 @@ void DockWindow::loadPanels(const DockPageView* page)
     TRACEFUNC;
 
     for (DockPanelView* panel : page->panels()) {
-        if (DockPanelView* destinationPanel = findDestinationForPanel(page, panel)) {
+        if (DockPanelView* destinationPanel = page->findPanelForTab(panel)) {
             addPanelAsTab(panel, destinationPanel);
             continue;
         }
-        addDock(panel, panel->location());
+
+        const Location location = panel->location();
+        const bool isSideLocation = location == Location::Left || location == Location::Right;
+        addDock(panel, location, isSideLocation ? page->centralDock() : nullptr);
     }
 
     for (Location location : POSSIBLE_LOCATIONS) {
@@ -435,16 +461,6 @@ void DockWindow::alignTopLevelToolBars(const DockPageView* page)
     lastCentralToolBar->setMinimumWidth(lastCentralToolBar->contentWidth() + deltaForLastCentralToolBar);
 }
 
-DockPanelView* DockWindow::findDestinationForPanel(const DockPageView* page, const DockPanelView* panel) const
-{
-    for (DockPanelView* destinationPanel : page->panels()) {
-        if (destinationPanel->isTabAllowed(panel)) {
-            return destinationPanel;
-        }
-    }
-    return nullptr;
-}
-
 void DockWindow::addDock(DockBase* dock, Location location, const DockBase* relativeTo)
 {
     TRACEFUNC;
@@ -453,7 +469,7 @@ void DockWindow::addDock(DockBase* dock, Location location, const DockBase* rela
 
     KDDockWidgets::DockWidgetBase* relativeDock = relativeTo ? relativeTo->dockWidget() : nullptr;
 
-    auto visibilityOption = dock->isVisible() ? KDDockWidgets::InitialVisibilityOption::StartVisible
+    auto visibilityOption = dock->defaultVisibility() ? KDDockWidgets::InitialVisibilityOption::StartVisible
                             : KDDockWidgets::InitialVisibilityOption::StartHidden;
 
     KDDockWidgets::InitialOption options(visibilityOption, dock->preferredSize());
@@ -465,7 +481,7 @@ void DockWindow::addPanelAsTab(DockPanelView* panel, DockPanelView* destinationP
 {
     registerDock(panel);
 
-    if (panel->isVisible()) {
+    if (panel->defaultVisibility()) {
         destinationPanel->addPanelAsTab(panel);
         destinationPanel->setCurrentTabIndex(0);
     }
@@ -495,7 +511,7 @@ void DockWindow::handleUnknownDock(const DockPageView* page, DockBase* unknownDo
         return;
     }
 
-    if (DockPanelView* destinationPanel = findDestinationForPanel(page, unknownPanel)) {
+    if (DockPanelView* destinationPanel = page->findPanelForTab(unknownPanel)) {
         addPanelAsTab(unknownPanel, destinationPanel);
         return;
     }
@@ -538,6 +554,8 @@ bool DockWindow::doLoadPage(const QString& uri, const QVariantMap& params)
         return false;
     }
 
+    newPage->setVisible(true);
+
     loadPageContent(newPage);
     restorePageState(newPage);
     initDocks(newPage);
@@ -545,7 +563,9 @@ bool DockWindow::doLoadPage(const QString& uri, const QVariantMap& params)
     newPage->setParams(params);
 
     m_currentPage = newPage;
-    m_currentPage->setVisible(true);
+
+    connect(m_currentPage, &DockPageView::layoutRequested,
+            this, &DockWindow::forceLayout, Qt::UniqueConnection);
 
     return true;
 }
@@ -661,6 +681,11 @@ bool DockWindow::checkLayoutIsCorrupted() const
     return false;
 }
 
+void DockWindow::forceLayout()
+{
+    m_mainWindow->layoutEqually();
+}
+
 QByteArray DockWindow::windowState() const
 {
     TRACEFUNC;
@@ -697,7 +722,13 @@ void DockWindow::initDocks(DockPageView* page)
 {
     TRACEFUNC;
 
+    //! before init we should correct toolbars sizes
+    adjustContentForAvailableSpace(page);
+
     for (DockToolBarView* toolbar : m_toolBars.list()) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+        toolbar->setParentItem(this);
+#endif
         toolbar->init();
     }
 
@@ -708,24 +739,27 @@ void DockWindow::initDocks(DockPageView* page)
 
     alignTopLevelToolBars(page);
 
+    if (!m_pageConnections.contains(page)) {
+        m_pageConnections[page] = new UniqueConnectionHolder(page, this);
+    }
+
+    UniqueConnectionHolder* holder = m_pageConnections[page];
+
     for (DockToolBarView* toolbar : topLevelToolBars(page)) {
-        connect(toolbar, &DockToolBarView::floatingChanged, this, [this, page]() {
-            alignTopLevelToolBars(page);
-        }, Qt::UniqueConnection);
+        connect(toolbar, &DockToolBarView::floatingChanged,
+                holder, &UniqueConnectionHolder::alignTopLevelToolBars, Qt::UniqueConnection);
 
-        connect(toolbar, &DockToolBarView::contentSizeChanged, this, [this, page]() {
-            alignTopLevelToolBars(page);
-        }, Qt::UniqueConnection);
+        connect(toolbar, &DockToolBarView::contentSizeChanged,
+                holder, &UniqueConnectionHolder::alignTopLevelToolBars, Qt::UniqueConnection);
 
-        connect(toolbar, &DockToolBarView::visibleChanged, this, [this, page]() {
-            alignTopLevelToolBars(page);
-        }, Qt::UniqueConnection);
+        connect(toolbar, &DockToolBarView::visibleChanged,
+                holder, &UniqueConnectionHolder::alignTopLevelToolBars, Qt::UniqueConnection);
     }
 }
 
-void DockWindow::adjustContentForAvailableSpace()
+void DockWindow::adjustContentForAvailableSpace(DockPageView* page)
 {
-    if (!m_currentPage) {
+    if (!page) {
         return;
     }
 
@@ -754,8 +788,8 @@ void DockWindow::adjustContentForAvailableSpace()
                 if (!dock->isCompact()) {
                     dock->setIsCompact(true);
 
-                    //! NOTE: as soon as we have compacted the first found dock - we finish the work so that the view is redrawn
-                    break;
+                    width -= dock->nonCompactWidth();
+                    width += dock->width();
                 }
             }
         } else {
@@ -778,8 +812,8 @@ void DockWindow::adjustContentForAvailableSpace()
 
     QList<DockBase*> topLevelToolBarsDocks;
 
-    for (DockToolBarView* toolBar : topLevelToolBars(m_currentPage)) {
-        if (!toolBar->dockWidget()->isFloating()) {
+    for (DockToolBarView* toolBar : topLevelToolBars(page)) {
+        if (!toolBar->dockWidget()->isFloating() && toolBar->isVisible()) {
             topLevelToolBarsDocks << toolBar;
         }
     }
@@ -826,3 +860,5 @@ QList<DockToolBarView*> DockWindow::topLevelToolBars(const DockPageView* page) c
 
     return toolBars;
 }
+
+#include "dockwindow.moc"
