@@ -35,8 +35,8 @@ class IProcess : MODULE_EXPORT_INTERFACE
 public:
     virtual ~IProcess() = default;
 
-    virtual int execute(const std::string& program, const std::vector<std::string>& arguments) = 0;
-    virtual bool startDetached(const std::string& program, const std::vector<std::string>& arguments) = 0;
+    virtual int execute(const std::string& program, const std::vector<std::string>& arguments = {}) = 0;
+    virtual bool startDetached(const std::string& program, const std::vector<std::string>& arguments = {}) = 0;
 };
 }
 
