@@ -69,6 +69,8 @@ using voice_layer_idx_t = uint_fast8_t;
 using staff_layer_idx_t = uint_fast16_t;
 using layer_idx_t = size_t;
 
+static constexpr duration_t INFINITE_DURATION = std::numeric_limits<duration_t>::max();
+
 struct TimestampAndDuration {
     timestamp_t timestamp = 0;
     duration_t duration = 0;
@@ -197,7 +199,6 @@ enum class ArticulationType : signed char {
     SoftAccent,
 
     LaissezVibrer,
-    LetRing,
 
     Subito,
 
@@ -292,12 +293,35 @@ enum class ArticulationType : signed char {
     Slap,
     Pop,
 
+    LeftHandTapping,
+    RightHandTapping,
+
     ContinuousGlissando,
 
+    // Handbells
+    MalletBellOnTable,
+    MalletBellSuspended,
+    MalletLift,
+    Pluck,
+    PluckLift,
+    Gyro,
+    Martellato,
+    MartellatoLift,
+    HandMartellato,
+    MutedMartellato,
+    ThumbDamp,
+    BrushDamp,
+    Ring,
+    RingTouch,
+    SingingBell,
+    SingingVibrate,
+    Swing,
+    Echo,
+
     // multi-note articulations
     Trill,
     Crescendo,
-    Decrescendo,
+    Diminuendo,
     DiscreteGlissando,
     Legato,
     Pedal,
@@ -315,10 +339,10 @@ using ArticulationTypeSet = std::unordered_set<ArticulationType>;
 
 inline bool isMultiNoteArticulation(const ArticulationType type)
 {
-    static const ArticulationTypeSet MULTI_TYPES = {
+    static const ArticulationTypeSet MULTI_TYPES {
         ArticulationType::Trill,
         ArticulationType::Crescendo,
-        ArticulationType::Decrescendo,
+        ArticulationType::Diminuendo,
         ArticulationType::DiscreteGlissando,
         ArticulationType::Legato,
         ArticulationType::Pedal,
@@ -406,6 +430,9 @@ constexpr inline dynamic_level_t dynamicLevelFromType(const DynamicType type)
 
 struct ArrangementPattern
 {
+    duration_percentage_t durationFactor = 0;
+    duration_percentage_t timestampOffset = 0;
+
     ArrangementPattern() = default;
 
     ArrangementPattern(duration_percentage_t _durationFactor, duration_percentage_t _timestampOffset)
@@ -413,9 +440,6 @@ struct ArrangementPattern
     {
     }
 
-    duration_percentage_t durationFactor = 0;
-    duration_percentage_t timestampOffset = 0;
-
     bool operator==(const ArrangementPattern& other) const
     {
         return durationFactor == other.durationFactor
@@ -429,6 +453,8 @@ struct PitchPattern
 {
     using PitchOffsetMap = PitchCurve;
 
+    PitchOffsetMap pitchOffsetMap;
+
     PitchPattern() = default;
 
     PitchPattern(size_t size, percentage_t step, pitch_level_t defaultValue)
@@ -438,8 +464,6 @@ struct PitchPattern
         }
     }
 
-    PitchOffsetMap pitchOffsetMap;
-
     pitch_level_t maxAmplitudeLevel() const
     {
         return pitchOffsetMap.maxAmplitudeLevel();
@@ -459,6 +483,8 @@ struct ExpressionPattern
 {
     using DynamicOffsetMap = ExpressionCurve;
 
+    ExpressionCurve dynamicOffsetMap;
+
     ExpressionPattern() = default;
 
     ExpressionPattern(size_t size, percentage_t step, dynamic_level_t defaultValue)
@@ -468,8 +494,6 @@ struct ExpressionPattern
         }
     }
 
-    ExpressionCurve dynamicOffsetMap;
-
     dynamic_level_t maxAmplitudeLevel() const
     {
         return dynamicOffsetMap.maxAmplitudeLevel();
@@ -485,16 +509,16 @@ using ExpressionPatternList = std::vector<ExpressionPattern>;
 
 struct ArticulationPatternSegment
 {
+    ArrangementPattern arrangementPattern;
+    PitchPattern pitchPattern;
+    ExpressionPattern expressionPattern;
+
     ArticulationPatternSegment() = default;
 
     ArticulationPatternSegment(ArrangementPattern&& arrangement, PitchPattern&& pitch, ExpressionPattern&& expression)
         : arrangementPattern(arrangement), pitchPattern(pitch), expressionPattern(expression)
     {}
 
-    ArrangementPattern arrangementPattern;
-    PitchPattern pitchPattern;
-    ExpressionPattern expressionPattern;
-
     bool operator==(const ArticulationPatternSegment& other) const
     {
         return arrangementPattern == other.arrangementPattern
@@ -509,12 +533,16 @@ struct ArticulationsProfile
 {
     std::vector<ArticulationFamily> supportedFamilies;
 
-    const ArticulationPattern& pattern(const ArticulationType type) const
+    const ArticulationPattern& pattern(const ArticulationType type,
+                                       const ArticulationType fallback = ArticulationType::Undefined) const
     {
         auto search = m_patterns.find(type);
+        if (search == m_patterns.cend() && fallback != ArticulationType::Undefined) {
+            search = m_patterns.find(fallback);
+        }
 
         if (search == m_patterns.cend()) {
-            static ArticulationPattern emptyPattern;
+            static const ArticulationPattern emptyPattern;
             return emptyPattern;
         }
 
@@ -559,6 +587,14 @@ using ArticulationsProfilePtr = std::shared_ptr<ArticulationsProfile>;
 
 struct ArticulationMeta
 {
+    ArticulationType type = ArticulationType::Undefined;
+    ArticulationPattern pattern;
+
+    timestamp_t timestamp = 0;
+    duration_t overallDuration = 0;
+    pitch_level_t overallPitchChangesRange = 0;
+    dynamic_level_t overallDynamicChangesRange = 0;
+
     ArticulationMeta() = default;
 
     ArticulationMeta(const ArticulationType _type)
@@ -579,14 +615,6 @@ struct ArticulationMeta
         overallDynamicChangesRange(overallDynamicRange)
     {}
 
-    ArticulationType type = ArticulationType::Undefined;
-    ArticulationPattern pattern;
-
-    timestamp_t timestamp = 0;
-    duration_t overallDuration = 0;
-    pitch_level_t overallPitchChangesRange = 0;
-    dynamic_level_t overallDynamicChangesRange = 0;
-
     bool operator==(const ArticulationMeta& other) const
     {
         return type == other.type
@@ -596,11 +624,31 @@ struct ArticulationMeta
                && overallPitchChangesRange == other.overallPitchChangesRange
                && overallDynamicChangesRange == other.overallDynamicChangesRange;
     }
+
+    bool hasStart() const
+    {
+        return overallDuration > 0;
+    }
+
+    bool hasEnd() const
+    {
+        return overallDuration != mpe::INFINITE_DURATION;
+    }
 };
 
 using ArticulationMetaMap = SharedHashMap<ArticulationType, ArticulationMeta>;
 
 struct ArticulationAppliedData {
+    ArticulationMeta meta;
+
+    ArticulationPatternSegment appliedPatternSegment;
+
+    duration_percentage_t occupiedFrom = 0;
+    duration_percentage_t occupiedTo = HUNDRED_PERCENT;
+
+    pitch_level_t occupiedPitchChangesRange = 0;
+    dynamic_level_t occupiedDynamicChangesRange = 0;
+
     ArticulationAppliedData() = default;
 
     explicit ArticulationAppliedData(ArticulationMeta&& _meta,
@@ -619,6 +667,16 @@ struct ArticulationAppliedData {
         updateOccupiedRange(_occupiedFrom, _occupiedTo);
     }
 
+    bool operator==(const ArticulationAppliedData& other) const
+    {
+        return meta == other.meta
+               && appliedPatternSegment == other.appliedPatternSegment
+               && occupiedFrom == other.occupiedFrom
+               && occupiedTo == other.occupiedTo
+               && occupiedPitchChangesRange == other.occupiedPitchChangesRange
+               && occupiedDynamicChangesRange == other.occupiedDynamicChangesRange;
+    }
+
     void updateOccupiedRange(const duration_percentage_t from, const duration_percentage_t to)
     {
         occupiedFrom = from;
@@ -647,26 +705,6 @@ struct ArticulationAppliedData {
             appliedPatternSegment = meta.pattern.cbegin()->second;
         }
     }
-
-    ArticulationMeta meta;
-
-    ArticulationPatternSegment appliedPatternSegment;
-
-    duration_percentage_t occupiedFrom = 0;
-    duration_percentage_t occupiedTo = HUNDRED_PERCENT;
-
-    pitch_level_t occupiedPitchChangesRange = 0;
-    dynamic_level_t occupiedDynamicChangesRange = 0;
-
-    bool operator==(const ArticulationAppliedData& other) const
-    {
-        return meta == other.meta
-               && appliedPatternSegment == other.appliedPatternSegment
-               && occupiedFrom == other.occupiedFrom
-               && occupiedTo == other.occupiedTo
-               && occupiedPitchChangesRange == other.occupiedPitchChangesRange
-               && occupiedDynamicChangesRange == other.occupiedDynamicChangesRange;
-    }
 };
 
 struct ArticulationMap : public SharedHashMap<ArticulationType, ArticulationAppliedData>
