@@ -23,22 +23,19 @@
 
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
-#include <QTimer>
 #include <QEventLoop>
 #include <QUrl>
 
-#include "log.h"
 #include "networkerrors.h"
 
 using namespace muse;
 using namespace muse::network;
 
-static constexpr int NET_TIMEOUT_MS = 60000;
-
 NetworkManager::NetworkManager(QObject* parent)
     : QObject(parent)
 {
     m_manager = new QNetworkAccessManager(this);
+    m_manager->setTransferTimeout(); // Use Qt's default timeout (30s)
 }
 
 NetworkManager::~NetworkManager()
@@ -81,8 +78,6 @@ Ret NetworkManager::del(const QUrl& url, IncomingDevice* incomingData, const Req
 Ret NetworkManager::execRequest(RequestType requestType, const QUrl& url, IncomingDevice* incomingData, OutgoingDevice* outgoingData,
                                 const RequestHeaders& headers)
 {
-    LOGI() << "Start: " << requestTypeToString(requestType) << ", url: " << url.toString();
-
     if (outgoingData && outgoingData->device()) {
         if (!openDevice(outgoingData->device(), QIODevice::ReadOnly)) {
             return make_ret(Err::FiledOpenIODeviceRead);
@@ -104,12 +99,12 @@ Ret NetworkManager::execRequest(RequestType requestType, const QUrl& url, Incomi
         _headers = configuration()->defaultHeaders();
     }
 
-    for (QNetworkRequest::KnownHeaders knownHeader: _headers.knownHeaders.keys()) {
-        request.setHeader(knownHeader, _headers.knownHeaders[knownHeader]);
+    for (auto it = _headers.knownHeaders.cbegin(); it != _headers.knownHeaders.cend(); ++it) {
+        request.setHeader(it.key(), it.value());
     }
 
-    for (const QByteArray& rawHeader: _headers.rawHeaders.keys()) {
-        request.setRawHeader(rawHeader, _headers.rawHeaders[rawHeader]);
+    for (auto it = _headers.rawHeaders.cbegin(); it != _headers.rawHeaders.cend(); ++it) {
+        request.setRawHeader(it.key(), it.value());
     }
 
     m_progress.start();
@@ -124,11 +119,7 @@ Ret NetworkManager::execRequest(RequestType requestType, const QUrl& url, Incomi
         prepareReplyReceive(reply, m_incomingData);
     }
 
-    Ret ret = waitForReplyFinished(reply, NET_TIMEOUT_MS);
-    if (!ret) {
-        LOGE() << ret.toString();
-    }
-
+    Ret ret = waitForReplyFinished(reply);
     m_progress.finish(ret);
 
     if (reply) {
@@ -142,8 +133,6 @@ Ret NetworkManager::execRequest(RequestType requestType, const QUrl& url, Incomi
     closeDevice(m_incomingData);
     m_incomingData = nullptr;
 
-    LOGI() << "Finish: " << requestTypeToString(requestType) << ", url: " << url.toString();
-
     return ret;
 }
 
@@ -179,6 +168,7 @@ QNetworkReply* NetworkManager::receiveReply(RequestType requestType, const QNetw
     }
     }
 
+    UNREACHABLE;
     return nullptr;
 }
 
@@ -193,7 +183,6 @@ void NetworkManager::abort()
         m_reply->abort();
     }
 
-    m_isAborted = true;
     m_progress.finish(make_ret(Err::Abort));
 }
 
@@ -217,11 +206,6 @@ void NetworkManager::closeDevice(QIODevice* device)
     }
 }
 
-bool NetworkManager::isAborted() const
-{
-    return m_isAborted;
-}
-
 void NetworkManager::prepareReplyReceive(QNetworkReply* reply, IncomingDevice* incomingData)
 {
     if (incomingData) {
@@ -246,48 +230,28 @@ void NetworkManager::prepareReplyReceive(QNetworkReply* reply, IncomingDevice* i
 
 void NetworkManager::prepareReplyTransmit(QNetworkReply* reply)
 {
-    connect(reply, &QNetworkReply::uploadProgress, [this](const qint64 curr, const qint64 total) {
+    connect(reply, &QNetworkReply::uploadProgress, this, [this](qint64 curr, qint64 total) {
         m_progress.progress(curr, total);
     });
 }
 
-Ret NetworkManager::waitForReplyFinished(QNetworkReply* reply, int timeoutMs)
+Ret NetworkManager::waitForReplyFinished(QNetworkReply* reply)
 {
-    QTimer timeoutTimer;
-    timeoutTimer.setSingleShot(true);
-    m_isAborted = false;
+    IF_ASSERT_FAILED(reply) {
+        return make_ret(Err::UnknownError);
+    }
 
-    bool isTimeout = false;
-    connect(&timeoutTimer, &QTimer::timeout, this, [this, &isTimeout]() {
-        isTimeout = true;
-        abort();
-    });
-
-    auto restartTimeoutTimer = [&timeoutTimer, &isTimeout](qint64, qint64) {
-        if (!isTimeout) {
-            timeoutTimer.start();
-        }
-    };
-
-    connect(reply, &QNetworkReply::downloadProgress, this, restartTimeoutTimer);
-    connect(reply, &QNetworkReply::uploadProgress, this, restartTimeoutTimer);
+    if (reply->isFinished()) {
+        return errorFromReply(reply);
+    }
 
     QEventLoop loop;
     connect(reply, &QNetworkReply::finished, &loop, &QEventLoop::quit);
 
     m_reply = reply;
-    timeoutTimer.start(timeoutMs);
     loop.exec();
     m_reply = nullptr;
 
-    if (isTimeout) {
-        return make_ret(Err::Timeout);
-    }
-
-    if (isAborted()) {
-        return make_ret(Err::Abort);
-    }
-
     return errorFromReply(reply);
 }
 
@@ -299,7 +263,11 @@ Ret NetworkManager::errorFromReply(const QNetworkReply* reply) const
 
     Ret ret = muse::make_ok();
 
-    if (reply->error() != QNetworkReply::NoError) {
+    if (reply->error() == QNetworkReply::TimeoutError) {
+        ret = make_ret(Err::Timeout);
+    } else if (reply->error() == QNetworkReply::OperationCanceledError) {
+        ret = make_ret(Err::Abort);
+    } else if (reply->error() != QNetworkReply::NoError) {
         ret.setCode(static_cast<int>(Err::NetworkError));
     }
 
@@ -315,17 +283,3 @@ Ret NetworkManager::errorFromReply(const QNetworkReply* reply) const
 
     return ret;
 }
-
-String NetworkManager::requestTypeToString(RequestType type)
-{
-    switch (type) {
-    case GET_REQUEST: return u"GET";
-    case HEAD_REQUEST: return u"HEAD";
-    case POST_REQUEST: return u"POST";
-    case PUT_REQUEST: return u"PUT";
-    case PATCH_REQUEST: return u"PATCH";
-    case DELETE_REQUEST: return u"DELETE";
-    }
-
-    return String();
-}
