@@ -34,6 +34,11 @@ bool QmlLauncher::open(const QString& uri)
     return true;
 }
 
+bool QmlLauncher::openSync(const QString& uri)
+{
+    return interactive()->openSync(UriQuery(uri.toStdString())).ret;
+}
+
 bool QmlLauncher::openApp(const QString& uri)
 {
     interactive()->openApp(UriQuery(uri.toStdString()));
