@@ -43,16 +43,6 @@ using namespace muse::uicomponents;
 
 static const muse::actions::ActionCode ADD_INSTRUMENTS_ACTIONCODE("instruments");
 
-namespace mu::instrumentsscene {
-static QString notationToKey(const INotationPtr notation)
-{
-    std::stringstream stream;
-    stream << notation.get();
-
-    return QString::fromStdString(stream.str());
-}
-}
-
 LayoutPanelTreeModel::LayoutPanelTreeModel(QObject* parent)
     : QAbstractItemModel(parent)
 {
@@ -62,7 +52,7 @@ LayoutPanelTreeModel::LayoutPanelTreeModel(QObject* parent)
     m_selectionModel->setAllowedModifiers(Qt::ShiftModifier);
 
     connect(m_selectionModel, &ItemMultiSelectionModel::selectionChanged,
-            [this](const QItemSelection& selected, const QItemSelection& deselected) {
+            this, [this](const QItemSelection& selected, const QItemSelection& deselected) {
         setItemsSelected(deselected.indexes(), false);
         setItemsSelected(selected.indexes(), true);
 
@@ -94,7 +84,6 @@ LayoutPanelTreeModel::LayoutPanelTreeModel(QObject* parent)
 void LayoutPanelTreeModel::onMasterNotationChanged()
 {
     m_masterNotation = context()->currentMasterNotation();
-    initPartOrders();
 }
 
 void LayoutPanelTreeModel::onNotationChanged()
@@ -118,6 +107,11 @@ void LayoutPanelTreeModel::onNotationChanged()
     m_notationChangedWhileLoadingWasBlocked = false;
 }
 
+bool LayoutPanelTreeModel::isMasterScore() const
+{
+    return m_notation && m_notation->isMaster();
+}
+
 LayoutPanelTreeModel::~LayoutPanelTreeModel()
 {
     deleteItems();
@@ -161,24 +155,7 @@ bool LayoutPanelTreeModel::shouldShowSystemObjectLayers() const
 {
     // Only show system object staves in master score
     // TODO: extend system object staves logic to parts
-    return m_notation && m_notation->isMaster();
-}
-
-void LayoutPanelTreeModel::initPartOrders()
-{
-    m_sortedPartIdList.clear();
-
-    if (!m_masterNotation) {
-        return;
-    }
-
-    for (IExcerptNotationPtr excerpt : m_masterNotation->excerpts()) {
-        NotationKey key = notationToKey(excerpt->notation());
-
-        for (const Part* part : excerpt->notation()->parts()->partList()) {
-            m_sortedPartIdList[key] << part->id();
-        }
-    }
+    return isMasterScore();
 }
 
 void LayoutPanelTreeModel::onBeforeChangeNotation()
@@ -194,8 +171,6 @@ void LayoutPanelTreeModel::onBeforeChangeNotation()
             partIdList << item->id();
         }
     }
-
-    m_sortedPartIdList[notationToKey(m_notation)] = partIdList;
 }
 
 void LayoutPanelTreeModel::setLoadingBlocked(bool blocked)
@@ -236,7 +211,7 @@ void LayoutPanelTreeModel::setupPartsConnections()
     m_notation->parts()->systemObjectStavesChanged().onNotify(this, [this]() {
         m_shouldUpdateSystemObjectLayers = true;
         if (!m_isLoadingBlocked) {
-            updateSystemObjectLayers();
+            load();
         }
     });
 }
@@ -310,7 +285,11 @@ void LayoutPanelTreeModel::setupNotationConnections()
         updateSelectedRows();
     });
 
-    m_notation->undoStack()->changesChannel().onReceive(this, [this](const mu::engraving::ScoreChangesRange& changes) {
+    m_notation->undoStack()->changesChannel().onReceive(this, [this](const mu::engraving::ScoreChanges& changes) {
+        if (changes.isTextEditing) {
+            return;
+        }
+
         if (!m_layoutPanelVisible) {
             m_scoreChanged = true;
             return;
@@ -351,7 +330,7 @@ void LayoutPanelTreeModel::updateSelectedRows()
     }
 }
 
-void LayoutPanelTreeModel::onScoreChanged(const mu::engraving::ScoreChangesRange& changes)
+void LayoutPanelTreeModel::onScoreChanged(const mu::engraving::ScoreChanges& changes)
 {
     if (!m_rootItem) {
         return;
@@ -397,7 +376,11 @@ void LayoutPanelTreeModel::load()
     m_rootItem = new RootTreeItem(m_masterNotation, m_notation, this);
 
     async::NotifyList<const Part*> masterParts = m_masterNotation->parts()->partList();
-    sortParts(masterParts);
+    async::NotifyList<const Part*> excerptParts = m_notation->parts()->partList();
+    // For master score we should use score's order, for others - custom
+    if (!isMasterScore()) {
+        sortParts(masterParts, excerptParts);
+    }
 
     const bool showSystemObjectLayers = shouldShowSystemObjectLayers();
     const std::vector<Staff*>& systemObjectStaves = m_masterNotation->notation()->parts()->systemObjectStaves();
@@ -410,13 +393,21 @@ void LayoutPanelTreeModel::load()
     for (const Part* part : masterParts) {
         if (showSystemObjectLayers) {
             for (Staff* staff : part->staves()) {
-                if (muse::contains(systemObjectStaves, staff)) {
+                if (!staff->hasSystemObjectsBelowBottomStaff() && muse::contains(systemObjectStaves, staff)) {
                     m_rootItem->appendChild(buildSystemObjectsLayerItem(staff, systemObjects[staff]));
                 }
             }
         }
 
         m_rootItem->appendChild(buildMasterPartItem(part));
+
+        if (showSystemObjectLayers) {
+            for (Staff* staff : part->staves()) {
+                if (staff->hasSystemObjectsBelowBottomStaff()) {
+                    m_rootItem->appendChild(buildSystemObjectsLayerItem(staff, systemObjects[staff]));
+                }
+            }
+        }
     }
 
     endResetModel();
@@ -430,30 +421,32 @@ void LayoutPanelTreeModel::load()
     emit isAddingAvailableChanged(true);
 }
 
-void LayoutPanelTreeModel::sortParts(notation::PartList& parts)
+void LayoutPanelTreeModel::sortParts(notation::PartList& parts, notation::PartList& referenceParts)
 {
-    NotationKey key = notationToKey(m_notation);
+    // First collect ids of referenceParts to use in sorting further
 
-    if (!m_sortedPartIdList.contains(key)) {
-        return;
+    std::vector<ID> referenceIdOrder;
+    referenceParts.reserve(referenceParts.size());
+
+    for (const Part* part : referenceParts) {
+        referenceIdOrder.push_back(part->id());
     }
 
-    const QList<muse::ID>& sortedPartIdList = m_sortedPartIdList[key];
+    // Now we can define ordering function based on reference order
+    auto comparator = [&referenceIdOrder] (const Part* partA, const Part* partB) {
+        const ID& idA = partA->id();
+        const ID& idB = partB->id();
 
-    std::sort(parts.begin(), parts.end(), [&sortedPartIdList](const Part* part1, const Part* part2) {
-        int index1 = sortedPartIdList.indexOf(part1->id());
-        int index2 = sortedPartIdList.indexOf(part2->id());
+        auto itA = std::find(referenceIdOrder.begin(), referenceIdOrder.end(), idA);
+        auto itB = std::find(referenceIdOrder.begin(), referenceIdOrder.end(), idB);
 
-        if (index1 < 0) {
-            index1 = std::numeric_limits<int>::max();
-        }
+        auto idxA = std::distance(referenceIdOrder.begin(), itA);
+        auto idxB = std::distance(referenceIdOrder.begin(), itB);
 
-        if (index2 < 0) {
-            index2 = std::numeric_limits<int>::max();
-        }
+        return idxA < idxB;
+    };
 
-        return index1 < index2;
-    });
+    std::stable_sort(parts.begin(), parts.end(), comparator);
 }
 
 void LayoutPanelTreeModel::setLayoutPanelVisible(bool visible)
@@ -579,8 +572,32 @@ bool LayoutPanelTreeModel::moveRows(const QModelIndex& sourceParent, int sourceR
 
     int sourceFirstRow = sourceRow;
     int sourceLastRow = sourceRow + count - 1;
-    int destinationRow = (sourceLastRow > destinationChild || sourceParentItem != destinationParentItem)
-                         ? destinationChild : destinationChild + 1;
+
+    if (sourceRow < 0 || count <= 0 || destinationChild < 0) {
+        return false;
+    }
+
+    if (sourceRow == destinationChild) {
+        return false;
+    }
+
+    int destinationRow = (sourceLastRow > destinationChild) ? destinationChild : destinationChild + 1;
+
+    if (sourceParentItem && destinationParentItem) {
+        int maxRow = sourceParentItem->childCount() - 1;
+
+        if (sourceLastRow > maxRow || destinationRow > maxRow + 1) {
+            return false;
+        }
+    }
+
+    if (!sourceParentItem || !destinationParentItem) {
+        return false;
+    }
+
+    if (sourceFirstRow >= sourceParentItem->childCount()) {
+        return false;
+    }
 
     if (m_dragInProgress) {
         MoveParams params = sourceParentItem->buildMoveParams(sourceRow, count, destinationParentItem, destinationRow);
@@ -596,7 +613,10 @@ bool LayoutPanelTreeModel::moveRows(const QModelIndex& sourceParent, int sourceR
     sourceParentItem->moveChildren(sourceFirstRow, count, destinationParentItem, destinationRow, !m_dragInProgress);
     endMoveRows();
 
-    updateSystemObjectLayers();
+    if (!m_dragInProgress) {
+        updateSystemObjectLayers();
+    }
+
     updateRearrangementAvailability();
 
     return true;
@@ -626,7 +646,8 @@ void LayoutPanelTreeModel::endActiveDrag()
 
 void LayoutPanelTreeModel::changeVisibilityOfSelectedRows(bool visible)
 {
-    for (const QModelIndex& index : m_selectionModel->selectedIndexes()) {
+    const QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+    for (const QModelIndex& index : selectedIndexes) {
         changeVisibility(index, visible);
     }
 }
@@ -707,7 +728,7 @@ int LayoutPanelTreeModel::columnCount(const QModelIndex&) const
 
 QVariant LayoutPanelTreeModel::data(const QModelIndex& index, int role) const
 {
-    if (!index.isValid() && role != ItemRole) {
+    if (!index.isValid() || role != ItemRole) {
         return QVariant();
     }
 
@@ -872,13 +893,13 @@ void LayoutPanelTreeModel::updateMovingDownAvailability(bool isSelectionMovable,
     const AbstractLayoutPanelTreeItem* curItem = modelIndexToItem(lastSelectedRowIndex);
     bool lastSelectedIsSystemObjectLayer = curItem && curItem->type() == LayoutPanelItemType::ItemType::SYSTEM_OBJECTS_LAYER;
 
-    IF_ASSERT_FAILED(lastSelectedRowIndex.row() != 0 || !lastSelectedIsSystemObjectLayer) {
+    if (lastSelectedRowIndex.row() == 0 && lastSelectedIsSystemObjectLayer) {
         // Selecting/moving the top system object layer not allowed
         setIsMovingDownAvailable(false);
         return;
     }
 
-    int lastItemRowIndex = parentItem->childCount() - 1 - (hasControlItem ? 1 : 0) - (lastSelectedIsSystemObjectLayer ? 1 : 0);
+    int lastItemRowIndex = parentItem->childCount() - 1 - (hasControlItem ? 1 : 0);
 
     bool isRowInBoundaries = lastSelectedRowIndex.isValid() && lastSelectedRowIndex.row() < lastItemRowIndex;
 
@@ -887,7 +908,8 @@ void LayoutPanelTreeModel::updateMovingDownAvailability(bool isSelectionMovable,
 
 void LayoutPanelTreeModel::updateRemovingAvailability()
 {
-    QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+    const QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+
     bool isRemovingAvailable = !selectedIndexes.empty();
     const AbstractLayoutPanelTreeItem* parent = nullptr;
 
@@ -927,7 +949,8 @@ void LayoutPanelTreeModel::updateRemovingAvailability()
 
 void LayoutPanelTreeModel::updateSelectedItemsType()
 {
-    QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+    const QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+
     LayoutPanelItemType::ItemType selectedItemsType = LayoutPanelItemType::ItemType::UNDEFINED;
 
     for (const QModelIndex& index : selectedIndexes) {
@@ -948,7 +971,7 @@ void LayoutPanelTreeModel::updateIsAddingSystemMarkingsAvailable()
 {
     bool addingSysMarkAvailable = false;
 
-    if (!isAddingAvailable() || !m_notation->isMaster()) {
+    if (!isAddingAvailable() || !isMasterScore()) {
         addingSysMarkAvailable = false;
     } else {
         int systemLayerCount = 0;
@@ -997,18 +1020,11 @@ bool LayoutPanelTreeModel::warnAboutRemovingInstrumentsIfNecessary(int count)
         return true;
     }
 
-    return interactive()->warning(
-        //: Please omit `%n` in the translation in this case; it's only there so that you
-        //: have the possibility to provide translations with the correct numerus form,
-        //: i.e. to show "instrument" or "instruments" as appropriate.
-        muse::trc("layoutpanel", "Are you sure you want to delete the selected %n instrument(s)?", nullptr, count),
-
-        //: Please omit `%n` in the translation in this case; it's only there so that you
-        //: have the possibility to provide translations with the correct numerus form,
-        //: i.e. to show "instrument" or "instruments" as appropriate.
-        muse::trc("layoutpanel", "This will remove the %n instrument(s) from the full score and all part scores.", nullptr, count),
-
-        { IInteractive::Button::No, IInteractive::Button::Yes })
+    return interactive()->warningSync(
+        muse::trc("layoutpanel", "Are you sure you want to delete the selected instrument(s)?", nullptr, count),
+        muse::trc("layoutpanel", "This will remove the instrument(s) from the full score and all part scores.", nullptr, count),
+        { IInteractive::Button::No, IInteractive::Button::Yes }
+        )
            .standardButton() == IInteractive::Button::Yes;
 }
 
@@ -1116,8 +1132,9 @@ void LayoutPanelTreeModel::updateSystemObjectLayers()
 
         const int partRow = partItem->row();
         const int layerRow = layerItem->row();
+        const int correctLayerRow = layerItem->staff()->hasSystemObjectsBelowBottomStaff() ? partRow + 1 : partRow - 1;
 
-        if (layerRow != partRow - 1) {
+        if (layerRow != correctLayerRow) {
             beginMoveRows(QModelIndex(), layerRow, layerRow, QModelIndex(), partRow);
             m_rootItem->moveChildren(layerRow, 1, m_rootItem, partRow, false /*updateNotation*/);
             endMoveRows();
@@ -1134,7 +1151,7 @@ void LayoutPanelTreeModel::updateSystemObjectLayers()
             }
 
             AbstractLayoutPanelTreeItem* newItem = buildSystemObjectsLayerItem(staff, systemObjects[staff]);
-            int row = partItem->row();
+            int row = staff->hasSystemObjectsBelowBottomStaff() ? partItem->row() + 1 : partItem->row();
 
             beginInsertRows(QModelIndex(), row, row);
             m_rootItem->insertChild(newItem, row);
@@ -1187,7 +1204,8 @@ const Staff* LayoutPanelTreeModel::resolveNewSystemObjectStaff() const
         return part->staff(0);
     };
 
-    for (const QModelIndex& index : m_selectionModel->selectedIndexes()) {
+    const QModelIndexList selectedIndexes = m_selectionModel->selectedIndexes();
+    for (const QModelIndex& index : selectedIndexes) {
         const AbstractLayoutPanelTreeItem* item = modelIndexToItem(index);
         if (const Staff* staff = resolveStaff(item)) {
             return staff;
