@@ -669,11 +669,13 @@ void NotationViewInputController::mousePressEvent(QMouseEvent* event)
 
     EngravingItem* hitElement = nullptr;
     staff_idx_t hitStaffIndex = muse::nidx;
+    bool shouldSeek = true;
 
     DEFER {
-        EngravingItem* playbackStartElement = resolveStartPlayableElement();
-        if (playbackStartElement) {
-            playbackController()->seekElement(playbackStartElement);
+        if (shouldSeek) {
+            if (EngravingItem* playbackStartElement = resolveStartPlayableElement()) {
+                playbackController()->seekElement(playbackStartElement);
+            }
         }
     };
 
@@ -689,7 +691,7 @@ void NotationViewInputController::mousePressEvent(QMouseEvent* event)
 
     // note enter mode
     if (m_view->isNoteEnterMode()) {
-        handleClickInNoteInputMode(event);
+        handleClickInNoteInputMode(event, shouldSeek);
         return;
     }
 
@@ -746,13 +748,13 @@ void NotationViewInputController::mousePressEvent(QMouseEvent* event)
 
     // Misc
     if (button == Qt::LeftButton) {
-        handleLeftClick(ctx);
+        handleLeftClick(ctx, shouldSeek);
     } else if (button == Qt::RightButton) {
         handleRightClick(ctx);
     }
 }
 
-void NotationViewInputController::handleClickInNoteInputMode(QMouseEvent* event)
+void NotationViewInputController::handleClickInNoteInputMode(QMouseEvent* event, bool& shouldSeek)
 {
     const PointF logicPos = m_view->toLogical(event->pos());
 
@@ -766,6 +768,7 @@ void NotationViewInputController::handleClickInNoteInputMode(QMouseEvent* event)
     const bool replace = keyState & Qt::ShiftModifier;
     const bool insert = keyState & Qt::ControlModifier;
     dispatcher()->dispatch("put-note", ActionData::make_arg3<PointF, bool, bool>(logicPos, replace, insert));
+    shouldSeek = false;
 }
 
 bool NotationViewInputController::mousePress_considerGrip(const ClickContext& ctx)
@@ -927,7 +930,7 @@ bool NotationViewInputController::mousePress_considerDragOutgoingRange(const Cli
     return false;
 }
 
-void NotationViewInputController::handleLeftClick(const ClickContext& ctx)
+void NotationViewInputController::handleLeftClick(const ClickContext& ctx, bool& shouldSeek)
 {
     if (!ctx.hitElement || !ctx.hitElement->selected()) {
         return;
@@ -940,6 +943,11 @@ void NotationViewInputController::handleLeftClick(const ClickContext& ctx)
     }
 
     if (ctx.hitElement->isPlayable()) {
+        if (EngravingItem* playbackStartElement = resolveStartPlayableElement()) {
+            playbackController()->seekElement(playbackStartElement);
+            shouldSeek = false;
+        }
+
         playbackController()->playElements({ ctx.hitElement });
     }
 
