@@ -58,6 +58,7 @@ static constexpr SymId commonScoreSymbols[] = {
     SymId::metNote64thUp,
     SymId::metNote128thUp,
     SymId::metAugmentationDot,
+    SymId::tuplet3,
     SymId::restWholeLegerLine,
     SymId::restHalfLegerLine,
     SymId::restQuarter,
@@ -441,13 +442,12 @@ SpecialCharactersDialog::SpecialCharactersDialog(QWidget* parent)
     QSplitter* ws = new QSplitter;
     m_lws = new QListWidget;
 
-    QStringList keys;
-    std::vector<String> symbols = muse::keys(Smufl::smuflRanges());
-    for (const String& s : symbols) {
-        keys << s.toQString();
+    QStringList smuflRangeNames;
+    for (const auto& pair: Smufl::smuflRanges()) {
+        smuflRangeNames << pair.first.toQString();
     }
 
-    m_lws->addItems(keys);
+    m_lws->addItems(smuflRangeNames);
     m_lws->setCurrentRow(0);
 
     ws->addWidget(m_lws);
@@ -493,8 +493,12 @@ SpecialCharactersDialog::SpecialCharactersDialog(QWidget* parent)
         const TextBase* editedText = interaction->editedText();
         setFont(editedText->font());
     }
+}
 
+void SpecialCharactersDialog::showEvent(QShowEvent* event)
+{
     WidgetStateStore::restoreGeometry(this);
+    TopLevelDialog::showEvent(event);
 }
 
 void SpecialCharactersDialog::hideEvent(QHideEvent* event)
@@ -685,14 +689,12 @@ void SpecialCharactersDialog::populateCommon()
 
 void SpecialCharactersDialog::populateSmufl()
 {
-    int row = m_lws->currentRow();
-
-    QString key = muse::keys(Smufl::smuflRanges()).at(row).toQString();
-    QStringList smuflNames = Smufl::smuflRanges().at(key).toQStringList();
-
     m_pSmufl->clear();
-    for (const QString& name : smuflNames) {
-        auto symId = SymNames::symIdByName(name);
+
+    int row = m_lws->currentRow();
+    std::vector<SymId> symIds = std::next(Smufl::smuflRanges().begin(), row)->second;
+
+    for (SymId symId : symIds) {
         std::shared_ptr<Symbol> s = std::make_shared<Symbol>(gpaletteScore->dummy());
         s->setSym(symId, gpaletteScore->engravingFont());
         m_pSmufl->appendElement(s, SymNames::translatedUserNameForSymId(symId));
@@ -712,7 +714,7 @@ void SpecialCharactersDialog::populateUnicode()
         std::shared_ptr<FSymbol> fs = std::make_shared<FSymbol>(gpaletteScore->dummy());
         fs->setCode(code);
         fs->setFont(m_font);
-        m_pUnicode->appendElement(fs, QString("0x%1").arg(code, 5, 16, QLatin1Char('0')));
+        m_pUnicode->appendElement(fs, QString("0x%1").arg(static_cast<int>(code), 5, 16, QLatin1Char('0')));
     }
 }
 
